@@ -4,8 +4,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
-#
+# http://www.apache.org/licenses/LICENSE-2.0#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -31,6 +30,7 @@ import sys
 import threading
 import time
 import util
+import imp
 
 from locking_dict import LockingDict
 
@@ -43,25 +43,15 @@ except:
 class Connector(threading.Thread):
     """Checks the cluster for shards to tail.
     """
-    def __init__(self, address, oplog_checkpoint, backend_url, ns_set,
+    def __init__(self, address, oplog_checkpoint, target_url, ns_set,
                  u_key, auth_key, doc_manager=None, auth_username=None):
         file = inspect.getfile(inspect.currentframe())
         cmd_folder = os.path.realpath(os.path.abspath(os.path.split(file)[0]))
         if doc_manager is not None:
-            if (doc_manager[0] is '/') or 'C:\\' in doc_manager:
-                if ('win32' or 'win64') in sys.platform:
-                    shutil.copy(doc_manager, cmd_folder + "\\doc_manager.py")
-                else:
-                    shutil.copy(doc_manager, cmd_folder + "/doc_manager.py")
-            else:
-                if ('win32' or 'win64') in sys.platform:
-                    shutil.copy(cmd_folder + "\\doc_managers\\" + doc_manager,
-                                cmd_folder + "\\doc_manager.py")
-                else:
-                    shutil.copy(cmd_folder + "/doc_managers/" + doc_manager,
-                                cmd_folder + "/doc_manager.py")
+            doc_manager = imp.load_source('DocManager', doc_manager)
+        else:
+            from doc_manager import DocManager
         time.sleep(1)
-        from doc_manager import DocManager
         super(Connector, self).__init__()
 
         #can_run is set to false when we join the thread
@@ -74,12 +64,12 @@ class Connector(threading.Thread):
         self.address = address
 
         #The URL of the target system
-        self.backend_url = backend_url
+        self.target_url = target_url
 
         #The set of relevant namespaces to consider
         self.ns_set = ns_set
 
-        #The key that is a unique document identifier for the backend system.
+        #The key that is a unique document identifier for the target system.
         #Not necessarily the mongo unique key.
         self.u_key = u_key
 
@@ -92,14 +82,22 @@ class Connector(threading.Thread):
         #The set of OplogThreads created
         self.shard_set = {}
 
-        #Dict of OplogThread/timestmap pairs to record progress
+        #Dict of OplogThread/timestamp pairs to record progress
         self.oplog_progress = LockingDict()
 
         try:
-            if backend_url is None:
-                self.doc_manager = DocManager()
+            if target_url is None:
+                if doc_manager is None:
+                    self.doc_manager = DocManager(unique_key=u_key)
+                else:
+                    self.doc_manager = doc_manager.DocManager(unique_key=u_key)
             else:
-                self.doc_manager = DocManager(self.backend_url)
+                if doc_manager is None:
+                    self.doc_manager = DocManager(self.target_url,
+                                                  unique_key=u_key)
+                else:
+                    self.doc_manager = doc_manager.DocManager(self.target_url,
+                                                              unique_key=u_key)
         except SystemError:
             logging.critical("MongoConnector: Bad target system URL!")
             self.can_run = False
@@ -127,27 +125,28 @@ class Connector(threading.Thread):
                 return None
 
         # write to temp file
-        os.rename(self.oplog_checkpoint, self.oplog_checkpoint + '++')
+        os.rename(self.oplog_checkpoint, self.oplog_checkpoint + '.backup')
         dest = open(self.oplog_checkpoint, 'w')
-        source = open(self.oplog_checkpoint + '++', 'r')
+        source = open(self.oplog_checkpoint + '.backup', 'r')
 
         # for each of the threads write to file
         self.oplog_progress.acquire_lock()
-
-        for oplog, ts in self.oplog_progress.dict.items():
-            oplog_str = str(oplog)
-            timestamp = util.bson_ts_to_long(ts)
-            json_str = json.dumps([oplog_str, timestamp])
-            dest.write(json_str)
-
-        self.oplog_progress.release_lock()
-
-        dest.close()
-        source.close()
-        os.remove(self.oplog_checkpoint + '++')
+        try:
+            oplog_dict = self.oplog_progress.get_dict()
+            for oplog, ts in oplog_dict.items():
+                oplog_str = str(oplog)
+                timestamp = util.bson_ts_to_long(ts)
+                json_str = json.dumps([oplog_str, timestamp])
+                dest.write(json_str)
+        finally:
+            self.oplog_progress.release_lock()
+            dest.close()
+            source.close()
+            os.remove(self.oplog_checkpoint + '.backup')
 
     def read_oplog_progress(self):
-        """Reads oplog progress from file provided by user
+        """Reads oplog progress from file provided by user.
+        This method is only called once before any threads are spanwed.
         """
 
         if self.oplog_checkpoint is None:
@@ -171,32 +170,30 @@ class Connector(threading.Thread):
             source.close()
             return None
 
+        source.close()
+
         count = 0
+        oplog_dict = self.oplog_progress.get_dict()
         for count in range(0, len(data), 2):
             oplog_str = data[count]
             ts = data[count + 1]
-            self.oplog_progress.dict[oplog_str] = util.long_to_bson_ts(ts)
+            oplog_dict[oplog_str] = util.long_to_bson_ts(ts)
             #stored as bson_ts
 
-        source.close()
-
     def run(self):
         """Discovers the mongo cluster and creates a thread for each primary.
         """
         main_conn = pymongo.Connection(self.address)
-        shard_coll = main_conn['config']['shards']
-
         self.read_oplog_progress()
+        conn_type = None
 
-        if shard_coll.find().count() == 0:
+        try:
+            main_conn.admin.command("isdbgrid")
+        except pymongo.errors.OperationFailure:
+            conn_type = "REPLSET"
+
+        if conn_type == "REPLSET":
             #non sharded configuration
-            try:
-                main_conn.admin.command("isdbgrid")
-                logging.error('MongoConnector: mongos has no shards!')
-                self.doc_manager.stop()
-                return
-            except pymongo.errors.OperationFailure:
-                pass
             oplog_coll = main_conn['local']['oplog.rs']
 
             prim_admin = main_conn.admin
@@ -232,6 +229,7 @@ class Connector(threading.Thread):
         else:       # sharded cluster
             while self.can_run is True:
 
+                shard_coll = main_conn['config']['shards']
                 shard_cursor = shard_coll.find()
 
                 for shard_doc in shard_cursor:
@@ -290,11 +288,6 @@ if __name__ == '__main__':
 
     formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
 
-    fh = logging.FileHandler('mongo_connector_log.txt')
-    fh.setLevel(logging.INFO)
-    fh.setFormatter(formatter)
-    logger.addHandler(fh)
-
     ch = logging.StreamHandler()
     ch.setLevel(logging.INFO)
     ch.setFormatter(formatter)
@@ -308,13 +301,13 @@ if __name__ == '__main__':
     #mongos. For non sharded clusters, it can be the primary.
     parser.add_option("-m", "--main", action="store", type="string",
                       dest="main_addr", default="localhost:27217",
-                      help="""Specify the mongos address, which is a """
-                      """host:port pair, or for clusters with one shard,"""
-                      """the primary's adderess. For example, `-m """
-                      """localhost:27217` would be a valid argument"""
-                      """to `-m`. It is not necessary to specify """
-                      """double-quotes aroung the argument to `-m`. Don't """
-                      """ use quotes around the address.""")
+                      help="""Specify the main address, which is a"""
+                      """ host:port pair. For sharded clusters,  this"""
+                      """ should be the mongos address. For individual"""
+                      """ replica sets, supply the address of the"""
+                      """ primary. For example, `-m localhost:27217`"""
+                      """ would be a valid argument to `-m`. Don't use"""
+                      """ quotes around the address""")
 
     #-o is to specify the oplog-config file. This file is used by the system
     #to store the last timestamp read on a specific oplog. This allows for
@@ -334,12 +327,12 @@ if __name__ == '__main__':
                       """ the connector will miss some documents and behave"""
                       """incorrectly.""")
 
-    #-b is to specify the URL to the backend engine being used.
-    parser.add_option("-b", "--backend-url", action="store", type="string",
+    #-t is to specify the URL to the target system being used.
+    parser.add_option("-t", "--target-url", action="store", type="string",
                       dest="url", default=None,
-                      help="""Specify the URL to the backend engine being """
+                      help="""Specify the URL to the target system being """
                       """used. For example, if you were using Solr out of """
-                      """the box, you could use '-b """
+                      """the box, you could use '-t """
                       """ http://localhost:8080/solr' with the """
                       """ SolrDocManager to establish a proper connection."""
                       """ Don't use quotes around address."""
@@ -358,24 +351,34 @@ if __name__ == '__main__':
                       """ also ignoring the "system.indexes" collection in """
                       """any database.""")
 
-    #-u is to specify the uniqueKey used by the backend,
+    #-u is to specify the mongoDB field that will serve as the unique key
+    #for the target system,
     parser.add_option("-u", "--unique-key", action="store", type="string",
                       dest="u_key", default="_id", help=
-                      """Used to specify the uniqueKey used by the backend."""
+                      """Used to specify the mongoDB field that will serve"""
+                      """as the unique key for the target system"""
                       """The default is "_id", which can be noted by """
                       """  '-u _id'""")
 
     #-k is to specify the authentication key file. This file is used by mongos
     #to authenticate connections to the shards, and we'll use it in the oplog
     #threads.
-    parser.add_option("-k", "--keyFile", action="store", type="string",
+    parser.add_option("-f", "--password-file", action="store", type="string",
                       dest="auth_file", default=None, help=
-                      """Used to specify the path to the authentication key"""
-                      """file. This file is used by mongos to authenticate"""
-                      """ connections to the shards, and we'll use it in the"""
+                      """ Used to store the password for authentication."""
+                      """ Use this option if you wish to specify a"""
+                      """ username and password but don't want to"""
+                      """ type in the password. The contents of this"""
+                      """ file should be the password for the admin user.""")
+
+    #-p is to specify the password used for authentication.
+    parser.add_option("-p", "--password", action="store", type="string",
+                      dest="password", default=None, help=
+                      """ Used to specify the password."""
+                      """ This is used by mongos to authenticate"""
+                      """ connections to the shards, and in the"""
                       """ oplog threads. If authentication is not used, then"""
-                      """ this field can be left empty as the default """
-                      """ is None.""")
+                      """ this field can be left empty as the default """)
 
     #-a is to specify the username for authentication.
     parser.add_option("-a", "--admin-username", action="store", type="string",
@@ -388,10 +391,16 @@ if __name__ == '__main__':
     #-d is to specify the doc manager file.
     parser.add_option("-d", "--docManager", action="store", type="string",
                       dest="doc_manager", default=None, help=
-                      """Used to specify the file in the /doc_managers"""
-                      """folder that should be used as the doc manager."""
-                      """Absolute paths also supported. By default, it will"""
-                      """use the doc_manager_simulator.py file.""")
+                      """Used to specify the doc manager file that"""
+                      """ is going to be used. You should send the"""
+                      """ path of the file you want to be used."""
+                      """ By default, it will use the """
+                      """ doc_manager_simulator.py file. It is"""
+                      """ recommended that all doc manager files be"""
+                      """ kept in the doc_managers folder in"""
+                      """ mongo-connector. For more information"""
+                      """ about making your own doc manager,"""
+                      """ see Doc Manager section.""")
 
     (options, args) = parser.parse_args()
 
@@ -411,9 +420,12 @@ if __name__ == '__main__':
             key = file.read()
             re.sub(r'\s', '', key)
         except:
-            logger.error('Could not parse authentication file!')
+            logger.error('Could not parse password authentication file!')
             sys.exit(1)
 
+    if options.password is not None:
+        key = options.password
+
     ct = Connector(options.main_addr, options.oplog_config, options.url,
                    ns_set, options.u_key, key, options.doc_manager,
                    auth_username=options.admin_name)
